@@ -1,5 +1,5 @@
 ##############################################################################
-#    Copyright (c) 2012-2017 Russell V. Lenth                                #
+#    Copyright (c) 2012-2024 Russell V. Lenth                                #
 #                                                                            #
 #    This file is part of the emmeans package for R (*emmeans*)              #
 #                                                                            #
@@ -38,7 +38,12 @@
 #' 
 #' @param object,x an object of class \code{emm_list}
 #' @param ... additional arguments passed to corresponding \code{emmGrid} method
-#' @param which integer vector specifying which elements to select.
+#' @param which integer vector specifying which elements to select;
+#' if \code{NULL},
+#' we try to guess which elements make sense. Usually, this is all elements having 
+#' names that start with \sQuote{em} or \sQuote{ls},
+#' or the first element if no matches are found. However, in \code{coef.emm_list},
+#' these are the ones we \emph{exclude}.
 #' 
 #' @return a \code{list} of objects returned by the corresponding \code{emmGrid}
 #'   method (thus, often, another \code{emm_list} object). However, if
@@ -66,7 +71,7 @@ NULL
     if (length(rtn) == 1)   {
         rtn = rtn[[1]]
     }
-    else                    {
+    else if(length(rtn) != 0)                   {
         cls = ifelse(class(rtn[[1]])[1] == "emmGrid", "emm_list", "summary_eml")
         class(rtn) = c(cls, "list")
     }
@@ -87,12 +92,15 @@ str.emm_list = function(object, ...) {
 }
 
 
-# summary.emm_list et all take an argument which that allows doing a subset
+# summary.emm_list et al take an argument 'which' that allows doing a subset
 # Each returns a regular 'list'
 
 
 #' @export
 #' @method summary emm_list
+#' @return \code{summary.emm_list} returns an object
+#' of class \code{summary_eml}, which is a list of \code{summary_emm}
+#' objects.
 #' @rdname emm_list-object
 #' @order 13
 summary.emm_list <- function(object, ..., which = seq_along(object)) {
@@ -108,12 +116,16 @@ summary.emm_list <- function(object, ..., which = seq_along(object)) {
 
 #' @export
 summary.summary_eml = function(object, ...) object
+
 #' @export
-as.data.frame.summary_eml = function(x, ...) {
-    for (i in seq_along(x))
-        attr(x[[i]], "digits") = getOption("digits")
-    x
+#' @rdname emm_list-object
+#' @order 25
+#' @param row.names,optional Required arguments of \code{as.data.frame}, ignored
+#' @method as.data.frame summary_eml
+as.data.frame.summary_eml = function(x, row.names = NULL, optional = FALSE, which, ...) {
+    rbind(x, which = which)
 }
+
 #' @export
 print.summary_eml = function(x, ...) {
     attr(x, "class") = NULL
@@ -132,20 +144,37 @@ print.emm_list = function(x, ...) {
     print(summary(x, ...))
 }
 
+### Utility fcn to identify which elements' names start with "em" or "lm"
+### this only operates if which == NULL
+.guess.which = guess.which = function(object, which) {
+    if(!is.null(which))
+        return(which)
+    rtn = which(substr(paste0(names(object), "xx"), 1, 2) %in% c("em", "ls"))
+    if(length(rtn) == 0)
+        rtn = 1
+    rtn
+}
+
 #' @export
 #' @method contrast emm_list
 #' @rdname emm_list-object
 #' @order 3
-contrast.emm_list = function(object, ... , which = 1) {
-    .lapply(object[which], contrast, ...)
+contrast.emm_list = function(object, ... , which = NULL) {
+    which = .guess.which(object, which)
+    rtn = .lapply(object[which], contrast, ...)
+    names(rtn) = paste("contrasts of", names(rtn))
+    rtn
 }
 
 #' @export
 #' @method pairs emm_list
 #' @rdname emm_list-object
 #' @order 4
-pairs.emm_list = function(x, ..., which = 1) {
-    .lapply(x[which], pairs, ...)
+pairs.emm_list = function(x, ..., which = NULL) {
+    which = .guess.which(x, which)
+    rtn = .lapply(x[which], pairs, ...)
+    names(rtn) = paste("comparisons of", names(rtn))
+    rtn
 }
 
 #' @export
@@ -153,6 +182,7 @@ pairs.emm_list = function(x, ..., which = 1) {
 #' @rdname emm_list-object
 #' @order 6
 test.emm_list = function(object, ..., which = seq_along(object)) {
+    which = .guess.which(object, which)
     .lapply(object[which], test, ...)
 }
 
@@ -161,6 +191,7 @@ test.emm_list = function(object, ..., which = seq_along(object)) {
 #' @rdname emm_list-object
 #' @order 7
 confint.emm_list = function(object, ..., which = seq_along(object)) {
+    which = .guess.which(object, which)
     .lapply(object[which], confint, ...)
 }
 
@@ -168,7 +199,8 @@ confint.emm_list = function(object, ..., which = seq_along(object)) {
 #' @method coef emm_list
 #' @rdname emm_list-object
 #' @order 9
-coef.emm_list = function(object, ..., which = 2) {
+coef.emm_list = function(object, ..., which = NULL) {
+    which = - .guess.which(object, which)
     .lapply(object[which], coef, ...)
 }
 
@@ -179,14 +211,18 @@ coef.emm_list = function(object, ..., which = 2) {
 #' @order 8
 #' @note The \code{plot} method uses only the first element of \code{which}; the others are ignored.
 plot.emm_list = function(x, ..., which = 1) {
+    which = .guess.which(x, which)
     plot.emmGrid(x[[which[1]]], ...)
 }
 
 #' @rdname rbind.emmGrid
 #' @order 23
-#' @param which Integer vector of subset of elements to use; if missing, all are combined
+#' @param which Integer vector of subset of elements to use;
+#' if missing, we use all elements. 
 #' @return The \code{rbind} method for \code{emm_list} objects simply combines 
 #' the \code{emmGrid} objects comprising the first element of \code{...}.
+#' Note that the returned object is not yet summarized, so any \code{adjust}
+#' parameters apply to the combined \code{emmGrid}.
 #' @export
 #' @method rbind emm_list
 #' @examples
@@ -198,15 +234,77 @@ plot.emm_list = function(x, ..., which = 1) {
 rbind.emm_list = function(..., which, adjust = "bonferroni") {
     elobj = list(...)[[1]]
     if(!missing(which))
-        elobj = elobj[which]
+         elobj = elobj[which]
     class(elobj) = c("emm_list", "list")
     update(do.call(rbind.emmGrid, elobj), adjust = adjust)
 }
 
+#' @rdname rbind.emmGrid
+#' @order 33
+#' @return The \code{rbind} method for \code{summary_emm} objects (or a list thereof)
+#' returns a single \code{summary_emm} object. This combined object
+#' \emph{preserves} any adjusted P values or confidence limits in the
+#' original summaries, since those quantities have already been computed.
+#' @export
+#' @method rbind summary_emm
+rbind.summary_emm = function(..., which) {
+    slobj = list(...)
+    if(!all(sapply(slobj, \(z) inherits(z, "data.frame")))) {
+        # workaround to make tern.gee::lsmeans() work
+        slobj = lapply(slobj, \(z) if(inherits(z, "data.frame")) data.frame(z) else z)
+       return(do.call("rbind", slobj))
+    }
+    rbind.summary_eml(slobj, which = which)
+}
+
+#' 
+#' @export
+#' @method rbind summary_eml
+rbind.summary_eml = function(..., which) {
+    x = list(...)[[1]]
+    if(!missing(which))
+        x = x[which]
+    nms.lst = lapply(x, names)
+    bys = unique(do.call(c, lapply(x, \(z) attr(z, "by.vars"))))
+    pris = unique(do.call(c, lapply(x, \(z) attr(z, "pri.vars"))))
+    if (length(x) == 1) {
+        attr(x[[1]], "pri.vars") = c(pris, bys)
+        attr(x[[1]], "by.vars") = NULL
+        return (x[[1]])
+    }
+    nms = pris = union(bys, pris)
+    for (n in nms.lst)
+        nms = union(nms, n)
+    nums = setdiff(nms, pris)  # numeric columns
+    xx = lapply(x, function(df) {
+        d = data.frame(matrix(".", nrow = nrow(df), ncol = length(nms),
+                       dimnames = list(NULL, nms)))
+        d[, nums] = NA
+        d[, names(df)] = df
+        d
+    })
+    rtn = do.call("rbind", xx)
+    row.names(rtn) = NULL
+    class(rtn) = c("summary_emm", "data.frame")
+    attr(rtn, "pri.vars") = pris
+    attr(rtn, "estName") = attr(x[[1]], "estName")
+    mesg = otr.mesg = attr(x[[1]], "mesg")
+    for (i in 2:length(x)) {
+        mesg = intersect(mesg, attr(x[[i]], "mesg"))
+        otr.mesg = union(otr.mesg, attr(x[[i]], "mesg"))
+    }
+    otr.mesg = setdiff(otr.mesg, mesg)
+    if (length(otr.mesg) > 0) 
+        mesg = c(mesg, "The following messages apply only to some rows:",
+                 paste("*", otr.mesg))
+    attr(rtn, "mesg") = mesg
+    rtn
+}
+
 #' @export
 #' @rdname emm_list-object
 #' @order 24
-#' @return The \code{as.data.frame} method returns a single data frame via
+#' @return The \code{as.data.frame} methods return a single data frame via
 #' \code{as.data.frame(rbind(x))}.
 #' See also \code{\link{rbind.emm_list}} and \code{\link{as.data.frame.emmGrid}}
 #' @method as.data.frame emm_list
@@ -241,13 +339,15 @@ as.emm_list = function(object, ...) {
 
 
 ### Others we won't document
-
+#' @exportS3Method update emm_list
 update.emm_list = function(object, ...)
     update.emmGrid(object[[1]])
 
+#' @exportS3Method predict emm_list
 predict.emm_list = function(object, ...)
     predict.emmGrid(object[[1]], ...)
 
+#' @exportS3Method vcov emm_list
 vcov.emm_list = function(object, ...)
     vcov.emmGrid(object[[1]], ...)
 
