@@ -93,6 +93,9 @@ emmeans.list = function(object, specs, ...) {
 #'   over which EMMs are desired. \code{specs} may also be a \code{formula}
 #'   or a \code{list} (optionally named) of valid \code{spec}s. Use of formulas
 #'   is described in the Overview section below. 
+#'   Specifying \code{.} as the only factor name
+#'   creates a list of specifications for all model terms.
+#'   
 #'   \bold{Note:} We recommend \emph{against} using two-sided formulas; see the
 #'   note below for \code{contr}.
 #' @param by A character vector specifying the names of predictors to condition on.
@@ -170,6 +173,14 @@ emmeans.list = function(object, specs, ...) {
 #' \code{*} or \code{:} are needed in the formula to delineate names, but
 #' otherwise are ignored.
 #' 
+#' We now also allow using \code{.} in \code{specs}. If this is done, we run
+#' \code{\link{joint_tests}} on the side to determine all relevant model terms,
+#' then replace \code{specs} with a corresponding list of specifications. This is
+#' a convenience, but it can create a sizeable \code{emm_list} object and it is
+#' coded rather inefficiently. While it is permissible to include contrasts
+#' via a \code{contr} argument or formula left-hand-side, we recommend instead
+#' doing this in a follow-up test with \code{\link{contrast.emm_list}}.
+#' 
 #' In the special case where the mean (or weighted mean) of all the predictions
 #' is desired, specify \code{specs} as \code{~ 1} or \code{"1"}.
 #' 
@@ -211,6 +222,20 @@ emmeans.list = function(object, specs, ...) {
 #' \code{"cells"}, except nonempty cells are weighted equally and empty cells
 #' are ignored.
 #' 
+#' @section Counterfactuals:
+#' Counterfactual reference grids (see the documentation for \code{\link{ref_grid}})
+#' contain pairs of imputed and actual factor levels, and are handled in a special way.
+#' For starters, the \code{weights} argument is ignored and we always use 
+#' \code{"cells"} weights.
+#' Our understanding is that if factors \code{A, B} are specified as counterfactuals, 
+#' the marginal means for \code{A} should still be the same as if \code{A} were the only 
+#' counterfactual. Accordingly, in computing these marginal means, we
+#' exclude all cases where \code{B != actual_B}, because if \code{A} were the only
+#' counterfactual, \code{B} will stay at its actual level.
+#' We also take special pains to "remember" information about actual and 
+#' imputed levels of counterfactuals so that appropriate results are obtained when
+#' \code{emmeans} is applied to a previous \code{emmeans} result.
+#' 
 #' 
 #' @section Offsets:
 #' Unlike in \code{ref_grid}, an offset need not be scalar. If not enough values
@@ -260,6 +285,10 @@ emmeans.list = function(object, specs, ...) {
 #' # 'adjust' argument NOT ignored ...
 #' emmeans (warp.lm, ~ tension | wool, adjust = "sidak")
 #' 
+#' # Get all sets of EMMs for this model
+#' ( allsets <- emmeans(warp.lm, ".") )
+#' contrast(allsets, "eff")    # all effects
+#' 
 #' 
 #' \dontrun{
 #'   ### Offsets: Consider a silly example:
@@ -281,6 +310,16 @@ emmeans = function(object, specs, by = NULL,
             args$wt.nuis = ifelse(!missing(weights) && is.character(weights), weights, "equal")
         object = do.call(ref_grid, args)
     }
+    
+    # Check to see if we want all sets of means
+    specs = .parse.specs.for.all(object, specs, by)
+    if(!is.null(rtn <- attr(specs, "form.rtn"))) {
+        if(length(rtn$by) > 0)
+            by = rtn$by
+        if(length(rtn$lhs) > 0) 
+            contr = rtn$lhs
+    }
+    
     if (is.list(specs)) {
         return (emmeans.list(object, specs, by = by, 
                              contr = contr, weights = weights, 
@@ -315,6 +354,32 @@ emmeans = function(object, specs, by = NULL,
             warning("emmeans() results may be corrupted by removal of a nesting structure")
         }
         
+        ## Handle counterfactuals...
+        # cf.grid is either NULL, logical, or a "parent" grid that should replace RG
+        if(!is.null(cf.grid <- RG@misc$cf.grid)) {
+            weights = "cells"  # always use cells weights with counterfactuals
+            if (inherits(cf.grid, "emmGrid"))
+                RG = cf.grid
+            # which counterfactuals do we average over?
+            cf.ao = intersect(setdiff(names(RG@levels), facs), RG@roles$counterfactuals)
+            # zero-out weights for any cases where cf.ao levels differ from actual levels
+            for(f in cf.ao) {
+                fc = paste0("actual_", f)
+                excl = (RG@grid[ , f] != RG@grid[, fc])
+                RG@grid[excl, ".wgt."] = 0
+            }
+            # Fix up the returned grid to play along
+            cf.surv = intersect(facs, RG@roles$counterfactuals)
+            if(length(cf.surv) == 0) # no cfs remain
+                RG@misc$cf.grid = RG@roles$counterfactuals = NULL
+            else {
+                if(length(intersect(facs, paste0("actual_", cf.surv))) == length(cf.surv))
+                    RG@misc$cf.grid = TRUE
+                else
+                    RG@misc$cf.grid = RG   # save this as "parent" grid
+            }
+        }
+        
         # Ensure object is in standard order
         ord = .std.order(RG@grid, RG@levels) ###do.call(order, unname(RG@grid[rev(names(RG@levels))]))
         if(any(ord != seq_along(ord)))
@@ -445,8 +510,12 @@ emmeans = function(object, specs, by = NULL,
         RG@misc$by.vars = by
         RG@misc$avgd.over = union(RG@misc$avgd.over, avgd.over)
         RG@misc$methDesc = "emmeans"
-        RG@roles$predictors = names(levs)
-### Pass up 'new' as we're not changing its class  result = new("emmGrid", RG, linfct = linfct, levels = levs, grid = combs)
+        RG@roles$predictors = setdiff(names(levs), RG@roles$multresp)
+        # if ((length(RG@roles$multresp) > 0) && !(RG@roles$multresp %in% names(levs))) 
+        #     RG@roles$multresp = character(0)
+        # REPLACED BY:
+        RG@roles$multresp = intersect(RG@roles$multresp, names(levs))
+        
         result = as.emmGrid(RG)
         result@linfct = linfct
         result@levels = levs
@@ -468,6 +537,9 @@ emmeans = function(object, specs, by = NULL,
     
 
     if(!missing(contr)) { # return a list with emmeans and contrasts
+        # # skip doing contrasts when we have no primary factor
+        # if(length(result@misc$pri.vars) == 0) 
+        #     return(result)
         warn.contr = interactive() && (sys.parent() == 0) && is.character(contr) && 
             (length(result@misc$pri.vars) > 1)
         # NULL-out a bunch of arguments to not pass. 
